"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const fs = require("fs");
const _ = require("lodash");
const shelljs = require("shelljs");
const fetch = require("node-fetch");
const ModelsPack = require("@tradle/models-pack");
const _validateResource = require("@tradle/validate-resource");
const errors_1 = tslib_1.__importDefault(require("@tradle/errors"));
const empty_bucket_1 = require("./empty-bucket");
const models_1 = require("./models");
const errors_2 = require("./errors");
const prompts_1 = require("./prompts");
exports.get = (url) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const res = yield fetch(url);
    if (res.statusCode > 300) {
        throw new Error(res.statusText);
    }
    return yield res.json();
});
exports.getNamespace = ({ models, lenses }) => {
    const model = _.values(models)[0] || _.values(lenses)[0];
    return model && ModelsPack.getNamespace(model.id);
};
exports.pack = ({ namespace, models, lenses }) => {
    if (!(_.size(models) || _.size(lenses))) {
        throw new errors_2.Errors.InvalidInput('expected "models" and/or "lenses"');
    }
    if (!namespace) {
        namespace = exports.getNamespace({ models, lenses });
    }
    const modelsPack = ModelsPack.pack({ namespace, models, lenses });
    return modelsPack;
};
exports.validateResource = resource => _validateResource({
    models: models_1.models,
    resource
});
exports.prettify = obj => {
    return typeof obj === 'string' ? obj : JSON.stringify(obj, null, 2);
};
exports.isValidProjectPath = project => {
    return fs.existsSync(path.resolve(project, 'serverless.yml'));
};
exports.toEnvFile = obj => Object.keys(obj)
    .map(key => `${key}="${obj[key]}"`)
    .join('\n');
exports.confirmOrAbort = (msg) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const confirmed = yield prompts_1.confirm(msg);
    if (!confirmed) {
        throw new errors_2.Errors.UserAborted();
    }
});
const acceptAll = (item) => true;
exports.listStackResources = (aws, StackName, filter = acceptAll) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let resources = [];
    const opts = { StackName };
    while (true) {
        let { StackResourceSummaries, NextToken } = yield aws.cloudformation.listStackResources(opts).promise();
        resources = resources.concat(StackResourceSummaries.filter(filter));
        opts.NextToken = NextToken;
        if (!opts.NextToken)
            break;
    }
    return resources;
});
exports.listStackResourcesByType = (type, aws, stackName) => {
    return exports.listStackResources(aws, stackName, ({ ResourceType }) => type === ResourceType);
};
exports.listStackResourceIdsByType = (type, aws, stackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const resources = yield exports.listStackResourcesByType(type, aws, stackName);
    return resources.map(r => r.PhysicalResourceId);
});
exports.listStackBuckets = exports.listStackResourcesByType.bind(null, 'AWS::S3::Bucket');
exports.listStackBucketIds = exports.listStackResourceIdsByType.bind(null, 'AWS::S3::Bucket');
exports.listStackFunctions = exports.listStackResourcesByType.bind(null, 'AWS::Lambda::Function');
exports.listStackFunctionIds = exports.listStackResourceIdsByType.bind(null, 'AWS::Lambda::Function');
exports.destroyBucket = (aws, Bucket) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    yield empty_bucket_1.emptyBucket(aws.s3, Bucket);
    try {
        yield aws.s3.deleteBucket({ Bucket }).promise();
    }
    catch (err) {
        errors_1.default.ignore(err, [
            { code: 'ResourceNotFoundException' },
            { code: 'NoSuchBucket' },
        ]);
    }
});
exports.disableStackTerminationProtection = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.updateTerminationProtection({
        StackName,
        EnableTerminationProtection: false,
    }).promise();
});
exports.deleteStack = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    while (true) {
        try {
            return yield aws.cloudformation.deleteStack({ StackName }).promise();
        }
        catch (err) {
            if (!err.message.includes('UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS')) {
                throw err;
            }
            yield exports.wait(5000);
        }
    }
});
exports.awaitStackDelete = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackDeleteComplete', { StackName }).promise();
});
exports.awaitStackUpdate = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackUpdateComplete', { StackName }).promise();
});
exports.listStacks = (aws) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const listStacksOpts = {
        StackStatusFilter: ['CREATE_COMPLETE', 'UPDATE_COMPLETE']
    };
    let stackInfos = [];
    let keepGoing = true;
    while (keepGoing) {
        let { NextToken, StackSummaries } = yield aws.cloudformation.listStacks(listStacksOpts).promise();
        stackInfos = stackInfos.concat(StackSummaries.map(({ StackId, StackName }) => ({
            id: StackId,
            name: StackName
        })));
        listStacksOpts.NextToken = NextToken;
        keepGoing = !!NextToken;
    }
    return stackInfos;
});
exports.getApiBaseUrl = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const result = yield aws.cloudformation.describeStacks({ StackName }).promise();
    const { Outputs } = result.Stacks[0];
    const endpoint = Outputs.find(x => /^ServiceEndpoint/.test(x.OutputKey));
    return endpoint.OutputValue;
});
exports.splitCamelCase = str => str.split(/(?=[A-Z])/g);
exports.checkCommandInPath = cmd => {
    const { code } = shelljs.exec(`command -v ${cmd}`);
    if (code !== 0) {
        throw new errors_2.Errors.InvalidEnvironment(`Please install: ${cmd}`);
    }
};
exports.wait = millis => new Promise(resolve => setTimeout(resolve, millis));
exports.normalizeNodeFlags = flags => {
    if (!(flags.inspect || flags['inspect-brk'] || flags.debug || flags['debug-brk']))
        return;
    const nodeVersion = Number(process.version.slice(1, 2));
    if (nodeVersion >= 8) {
        if (flags.debug) {
            delete flags.debug;
            flags.inspect = true;
        }
        if (flags['debug-brk']) {
            delete flags['debug-brk'];
            flags['inspect-brk'] = true;
        }
    }
    else {
        if (flags.debug || flags['debug-brk']) {
            flags.inspect = true;
        }
    }
};
exports.pickNonNull = obj => _.pickBy(obj, val => val != null);
//# sourceMappingURL=utils.js.map